local function roundnum(num, idp)
	local mult = 10^(idp or 0)
	return math.floor(num * mult + 0.5) / mult
end

local function SecondsToClock(nSeconds)
	if nSeconds < 0.5 or nSeconds == inf then
		--return nil;
		return "00:00:00";
	else
		nHours = string.format("%02.f",math.floor(nSeconds/3600,0));
		nMins =  string.format("%02.f",math.floor(nSeconds/60 - (nHours*60),0));
		nSecs = string.format("%02.f",math.floor(nSeconds - nHours*3600 - nMins *60,0));
		time = nHours..":"..nMins..":"..nSecs;
		if time == "inf:nan:nan" then
			time = "00:00:00";
		end
		return time
	end
end

local function AddDragFrame(window)

	function window.Event:LeftDown()
		self.leftDown = true
		local mouse = Inspect.Mouse()
		self.originalXDiff = mouse.x - self:GetLeft()
		self.originalYDiff = mouse.y - self:GetTop()
		local left, top, right, bottom = window:GetBounds()
		window:ClearAll()
		window:SetPoint("TOPLEFT", UIParent, "TOPLEFT", left, top)
		window:SetWidth(right-left)
		window:SetHeight(bottom-top)
	end

	function window.Event:LeftUp()
		self.leftDown = false
	end

	function window.Event:LeftUpoutside()
		self.leftDown = false
	end

	function window.Event:MouseMove(x, y)
		if not self.leftDown then
			return
		end
		window:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x - self.originalXDiff, y - self.originalYDiff)
		XPRate_mouse_x = x - self.originalXDiff
		XPRate_mouse_y = y - self.originalYDiff
	end
end


local XPRate_mouse_x = 0;
local XPRate_mouse_y = 0;

XPRate_starttime = 0;
XPRate_xpgained = 0;
XPRate_xpgained_last = 0;
XPRate_xpnow = 0;
XPRate_playerid = nil;
XPRate_last_called = 0;

local main = UI.CreateContext("UI")
local box = UI.CreateFrame("Frame", "box", main)
local ticker = UI.CreateFrame("Text", "Ticker", box)
local ticker2 = UI.CreateFrame("Text", "Ticker2", box)
local ticker3 = UI.CreateFrame("Text", "Ticker3", box)
local ticker4 = UI.CreateFrame("Text", "Ticker4", box)

box:SetPoint("TOPLEFT",UIParent,"TOPLEFT",100,100)
box:SetWidth(400)
box:SetHeight(120)
box:SetAlpha(.6)
box:SetVisible(true)
box:SetBackgroundColor(0,0,0)
box:SetLayer(1)

ticker:SetText("No Data")
ticker:SetPoint("TOPRIGHT",box,"TOPRIGHT",0,0)
ticker:SetFontSize(18)
ticker:SetLayer(2)
ticker2:SetText("No Data")
ticker2:SetPoint("TOPRIGHT",box,"TOPRIGHT",0,30)
ticker2:SetFontSize(18)
ticker2:SetLayer(2)
ticker3:SetText("No Data")
ticker3:SetPoint("TOPRIGHT",box,"TOPRIGHT",0,60)
ticker3:SetFontSize(18)
ticker3:SetLayer(2)
ticker4:SetText("No Data")
ticker4:SetPoint("TOPRIGHT",box,"TOPRIGHT",0,90)
ticker4:SetFontSize(18)
ticker4:SetLayer(2)


function LoadEnd(id)
	if id == "XPRate" then
		XPRate_playerid = Inspect.Unit.Lookup("player")
	end
end

local function levelUp(units)
	if units[XPRate_playerid] then
		if Inspect.Experience() then
			XPRate_xpnow = Inspect.Experience().accumulated;
		end
	end
end

local function calcWidth()
	local w = 10;
	if ticker:GetWidth() > w then
		w = ticker:GetWidth();
	end
	if ticker2:GetWidth() > w then
		w = ticker2:GetWidth();
	end
	if ticker3:GetWidth() > w then
		w = ticker3:GetWidth();
	end
	if ticker4:GetWidth() > w then
		w = ticker4:GetWidth();
	end
	return w;
end

local function updateXP()
	local now = Inspect.Time.Real()
	if (now<XPRate_last_called+1.2) then
		return
	end
	XPRate_last_called = now
	if Inspect.Experience() then
		local elapsed = Inspect.Time.Real() - XPRate_starttime;
		local xpps = XPRate_xpgained/elapsed;
		local xpph = xpps * 3600;
		local _hide = 1;
		local _needed = 0;
		if Inspect.Experience().needed then
			_needed = Inspect.Experience().needed;
			_hide = 0;
		end
		local _accumulated = 0;
		if Inspect.Experience().accumulated then
			_accumulated = Inspect.Experience().accumulated;
		end
		
		if _hide > 0 then
			box:SetVisible(false)
		end
		local xptolevel = _needed-_accumulated;
		local etatolevel = xptolevel/xpps;
		local line2 = "Elapsed time: " .. SecondsToClock(elapsed) .. " (" .. XPRate_xpgained .. ")";
		local line3 = "XP / Std: " .. roundnum(xpph,0) .. " ETA to Level: " .. SecondsToClock(roundnum(etatolevel,0));
		ticker2:SetText(line2)
		ticker3:SetText(line3)
		box:SetWidth(calcWidth());
	end
end

local function sayXP()
	if Inspect.Experience() then
		local xpgained = Inspect.Experience().accumulated - XPRate_xpnow;
		XPRate_xpnow = Inspect.Experience().accumulated;
		XPRate_xpgained = XPRate_xpgained + xpgained;
		if(xpgained>0) then
			XPRate_xpgained_last = xpgained;
		end

		local elapsed = Inspect.Time.Real() - XPRate_starttime;
		local xpps = XPRate_xpgained/elapsed;
		local xpph = xpps * 3600;
		local xptolevel = Inspect.Experience().needed-Inspect.Experience().accumulated;
		local etatolevel = xptolevel/xpps;

		local line = "XP: " ..
		xptolevel ..
		" (" ..
		Inspect.Experience().accumulated ..
		"/" ..
		Inspect.Experience().needed .. ") R: " ..
		Inspect.Experience().rested ..
		" (" ..
		roundnum((Inspect.Experience().rested/Inspect.Experience().needed)*100,2) ..
		"%)";
		local line2 = "Elapsed time: " .. SecondsToClock(elapsed) .. " (" .. XPRate_xpgained .. ")";
		local line3 = "XP / Std: " .. roundnum(xpph,0) .. " ETA to Level: " .. SecondsToClock(roundnum(etatolevel,0));
		local showKills = 0;
		local line4 = "Kills to level (last xp: 0): 0";
		if(XPRate_xpgained>0) then
			line4 = "Kills to level (last xp: " .. XPRate_xpgained_last .. "): " .. tostring(math.ceil((Inspect.Experience().needed-Inspect.Experience().accumulated)/XPRate_xpgained_last));
		end
		-- print(line)
		-- print(line2)
		-- print(line3)
		-- print(line4);

		ticker:SetText(line)
		ticker2:SetText(line2)
		ticker3:SetText(line3)
		ticker4:SetText(line4)
		box:SetWidth(calcWidth());
	end
end

local function initXP()
	XPRate_starttime = Inspect.Time.Real();
	XPRate_xpnow = Inspect.Experience().accumulated;
	XPRate_xpgained = 0;
	XPRate_xpgained_last = 0;
	XPRate_last_called = 0;
end

local function XPsave()
	XPRate_x = XPRate_mouse_x
	XPRate_y = XPRate_mouse_y
end

local function XPload()

	if XPRate_x then
		XPRate_mouse_x = tonumber(XPRate_x)
	end

	if XPRate_y then
		XPRate_mouse_y = tonumber(XPRate_y)
	end


	--fade()
	box:ClearAll()
	box:SetPoint("TOPLEFT", main, "TOPLEFT", XPRate_mouse_x, XPRate_mouse_y)
	box:SetWidth(400)
	box:SetHeight(120)


end

AddDragFrame(box)

table.insert(Command.Slash.Register("xpreset"), {function (params)
		initXP()
		sayXP()
	end, "XPRate", "Slash command"})

table.insert(Event.Experience.Accumulated, {function (params)
		sayXP()
	end, "XPRate", "Experience changed"})

table.insert(Event.Unit.Detail.Level, {function (params)
		levelUp(params)
	end, "XPRate", "Addon Loaded"})

table.insert(Event.Unit.Availability.Full, {function (params)
		initXP();
		table.remove(Event.Unit.Availability.Full,1);
	end, "XPRate", "Unit Avainable Full"})

table.insert(Event.Addon.Load.End, {LoadEnd,"XPRate", "Init"})
table.insert(Event.Addon.SavedVariables.Save.Begin, {XPsave, "XPRate", "Save variables"})
table.insert(Event.Addon.SavedVariables.Load.End, {XPload, "XPRate", "Load variables"})
table.insert(Event.System.Update.Begin, {updateXP, "XPRate", "Update ETA"})